<p align="center">
  <img src="docs/sampo_logo.png" alt="Logo of SAMPO framework" width="480pt"/>
</p>

# SAMPO - Планировщик для оптимизации адаптивных производственных процессов

SAMPO - это система с открытым исходным кодом для адаптивного планирования производственных процессов. Этот фреймворк распространяется под лицензией 3-Clause BSD.

Он предоставляет инструментарий для генерации планов выполнения задач в рамках производственного процесса в условиях ограничений, накладываемых предметной областью. В основе SAMPO лежит комбинация метаэвристических, генетических и мультиагентных алгоритмов. Принимая на вход граф задач со связями между ними, ограничения на ресурсы, а также метрику оптимизации, планировщик позволяет в автоматическом режиме построить оптимальную последовательность задач и распределение ресурсов по задачам в соответствии с заданной метрикой.


## Установка SAMPO

Самый простой способ установить SAMPO -- использовать ``pypi`` (установщик пакетов для языка Python).

```
 $ pip install sampo
```

## Как использовать SAMPO

Фреймворк SAMPO за счет своей универсальности может быть применим для решения задач оптимизации планирования в условиях ресурсных ограничений из разных предметных областей. Ниже приводятся примеры его применения для планирования задач производственных процессов на синтетических и реальных данных.

### Планирование на синтетических данных

  1. Импорт модулей ``generator`` и ``scheduler``
  ```
  from sampo import generator
  from sampo import scheduler

  ```
  2. Генерация синтетического графа работ и планирование
  ```
  wg = generator.SimpleSynthetic().advanced_work_graph(works_count_top_border=2000, uniq_works=300, uniq_resources=100)
  contractors = [generator.get_contractor_by_wg(wg)]
    
  schedule = scheduler.HEFTScheduler().schedule(wg, contractors)
  ```

### Планирование на реальных данных

Загрузка данных о структуре графа производится посредством загрузки информации о структуре графа (преобразованной в формат JSON) и о ресурсных командах подрядчиков (преобразованной в формат JSON) с использованием методов ``WorkGraph.load()`` и ``Contractor.load()``

```
from sampo.schemas.graph import WorkGraph
from sampo.schemas.contractor import Contractor

# Загрузка объекта WorkGraph из подготовленного JSON файла
field_development_wg = WorkGraph.load("./", "field_development_tasks_structure")

# Загрузка объекта Contractor из подготовленного JSON файла
contractors = [Contractor.load("./", "field_development_contractor_info")]
```

Для оптимизации структуры графа задач используется модуль ``structurator`` библиотеки Sampo и его основной метод ``graph_restructuring()``. Построение плана выполнения работ для структурно оптимизированного графа задач производится с помощью вызова метода ``schedule()`` класса определенного типа планировщика, который выбирается среди ключевых алгоритмов планирования, также представленных в модуле ``schedule``

```
from sampo.schemas.graph import WorkGraph
from sampo.schemas.contractor import Contractor

# Оптимизация структуры объекта WorkGraph 
structured_wg = graph_restructuring(field_development_wg, use_lag_edge_optimization=True)

# Планирование задач процесса
schedule = HEFTScheduler().schedule(structured_wg, contractors)
```


## Поддержка проекта

Разработка поддерживается исследовательским центром «Сильный искусственный интеллект в промышленности» Университета ИТМО.

<img src="docs/AIM-Strong_Sign_Norm-01_Colors.svg" alt="Logo of Strong AI research center" width="480pt"/>
